;-=========================================================================;
;- Copyright (C) 2000 H-Peter Recktenwald, Berlin <phpr@snafu.de>
;- Read LICENSE file for full copyright information (GNU GPL)
;-
;-  file  	: ccd.asm
;-  created	: 15-jan-2000
;-  modified	: 05-mar-2000
;-  version	: 0.05
;-  assembler	: nasm 0.98
;-  description	: ccd ("Change CD rom"):
;-		:	un-mount cdrom, eject, wait for <enter>, re-mount
;-		: syntax:
;-		:	ccd [specialfile [mountpoint [filesys [options]]]]
;-		: defaults:
;-		: 	     /dev/cdrom   /cdrom      iso9660  0
;-		: options:
;-		: 	raw binary only, ascii coded string input left as
;-		:	an exercise to the user - not required for CD-dev.
;-		: exit code
;-		:	is from device open.
;-  author	: H-Peter Recktenwald, Berlin, <phpr@snafu.de>
;-  comment	: requires root (superuser) permissions! doesn't use stdin
;-		: but /dev/tty, instead, thus should be safe for suid(?).
;-		: tested in linux 2.2.14 + rtl 2.0, ok w/wo default
;-		: parameters and, with example parm "/dev/hdb /cd".
;-  source	: re linux kernel sources
;-  requires	: "asmutils 07+" by K.Boldyshev, http://linuxassembly.org
;-=========================================================================;
;-
;-

%include "system.inc"

; ioctl no.s
%assign	CDROMEJECT	0x5309
%assign CDROMCLOSETRAY	0x5319
; flags
%define MS_RDONLY	1
%define MS_MGC_VAL	0xc0ed0000

    CODESEG

START:
	pop eax			; arg.count
	pop ebx			; prg.name
	lea edi,[options]
	lea ebx,[byte edi+dftdev-options]
	lea ecx,[byte edi+mountp-options]
	lea edx,[byte edi+fstype-options]
	dec eax
	jz .eject		;? defaults only
	pop ebx			; specialfile
	dec eax
	jz .eject
	pop ecx			; mountp
	dec eax
	jz .eject
	pop edx			; fstype
	dec eax
	jz .eject
	pop edi			; options
.eject:
	push ecx		; mountpoint
	push ebx		; specialfile

	sys_umount 		; eject won't work on a mounted device

	sys_open EMPTY,O_RDONLY|O_NONBLOCK
	push eax		; save exit code
	test eax,eax
	js .exit		;? device error, try re-mounting

	sys_ioctl eax,CDROMEJECT
	push ebx		; device-fd

	push edx		; fstype
	sys_open stdi,O_RDWR	; aux. console

	xor edx,edx
	mov dl,ts.e-ts		; message
	sys_write eax,ts	; (time to change CD)
	mov dl,1
	sys_read		; read input until <enter>
	sys_close
	pop edx

	pop ebx
	sys_ioctl EMPTY,CDROMCLOSETRAY

.exit:	;edx=fstype,edi=options	; (re-)mount, ready
	pop ecx			; ernum
	pop ebx			; device
	xchg ecx,[esp]		; mountpoint
	sys_mount EMPTY,EMPTY,EMPTY,MS_RDONLY|MS_MGC_VAL
	pop ebx
	test ebx,ebx
	js sexit		;? ret device open error
	xor ebx,ebx		; noerr
sexit:
	sys_exit


; watch trailing <nul> bytes,
; change string contents to suit
stdi:	db '/dev/tty'
options:dd 0
dftdev:	db '/dev'
mountp:	db '/cdrom',NULL
fstype:	db 'iso9660',NULL
ts:	db '<enter> ..',NULL
.e:

    
END
;-
;-=========================================================================;
;- ccd.asm <eof>
